/*
 * out_yasapi.c
 * Copyright (C) 2015-2016 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#if 0 // {
#include <windows.h>
#include <shlobj.h>
#include "../winamp/out.h"

#define PI_VER2 "v1.11 beta"

#ifdef __alpha
#define PI_VER PI_VER2 " (AXP)"
#else
#define PI_VER PI_VER2 " (x86)"
#endif
#else // } {
#include <yasapi.h>
#endif // }

#define GETTIME(time) \
  (bAudioClock?time:GetTickCount())

int getwrittentime();
int getoutputtime();

int srate, numchan, bps, active;
volatile int writtentime, w_offset;
int start_t;
static int last_pause=0;

static wchar_t *path;
static Player player;
static int bAudioClock;

#if 0 // {
BOOL WINAPI _DllMainCRTStartup(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
{
  return TRUE;
}

Out_Module out;
#endif // }

static int GetAudioTime(void)
{
  return player.state<PLAYER_STATE_OPEN
      ?0
      :PlayerDispatchSync(&player,0,PlayerGetTime);
}

void config(HWND hwnd)
{
  HINSTANCE hInstance;

  DPRINTF(player.options.common.nDebug,0,"%s\n",__func__);
  hInstance=winampGetOutModule()->hDllInstance;
  ConfigDialog(&player,hInstance,hwnd);
}

void about(HWND hwnd)
{
  AboutDialog(hwnd);
}

void init()
{
  int nDebug=0;

  if (NULL==(path=wapath(YASAPI_PROPERTY_FILE)))
    goto path;

#if defined (YASAPI_DEBUG) // {
#if 1 // {
  nDebug=GetPrivateProfileIntW(
    YASAPI_GROUP_COMMON,      // _In_  LPCTSTR lpAppName,
    YASAPI_PROPERTY_DEBUG,    // _In_  LPCTSTR lpKeyName,
    OptionsCommonDefault()->nDebug,
                              // _In_  INT nDefault,
    path                      // _In_  LPCTSTR lpFileName
  );
#else // } {
  nDebug=3;
#endif // }
#endif // }

  if (nDebug)
    ConsoleOpen();

  DPRINTF(nDebug,0,"%s\n",__func__);
  DWPRINTF(nDebug,0,L"  \"%s\"\n",path);

  if (PlayerCreate(&player,path,nDebug)<0) {
    DMESSAGE(nDebug,"creating plugin");
    goto plugin;
  }

  return;
//cleanup:
plugin:
#if defined (YASAPI_GLOBAL_ALLOC) // {
  GlobalFree(path);
#else // } {
  free(path);
#endif // }
path:
  return;
}

void quit()
{
#if defined (YASAPI_DEBUG) // {
  int nDebug=player.options.common.nDebug;
  int nSleep=player.options.common.nSleep;
#endif // }

  DPRINTF(nDebug,0,"%s\n",__func__);
  PlayerDestroy(&player);

#if defined (YASAPI_DEBUG) // {
  if (nDebug) {
  DPRINTF(nDebug,0,"  sleeping %d ms ...\n",nSleep);
    Sleep(nSleep);
    ConsoleClose();
  }
#endif // }

#if defined (YASAPI_GLOBAL_ALLOC) // {
  GlobalFree(path);
#else // } {
  free(path);
#endif // }
}

static void reset(void)
{
  start_t = GETTIME(0);
  w_offset = writtentime = 0;
}

int open(int samplerate, int numchannels, int bitspersamp, int bufferlenms, int prebufferms)
{
  DPRINTF(player.options.common.nDebug,0,"%s\n",__func__);

#if 0 // {
  start_t=GetTickCount();
  w_offset = writtentime = 0;
#else // } {
  bAudioClock=player.options.common.bAudioClock;
  reset();
#endif // }
  active=1;
  numchan = numchannels;
  srate = samplerate;
  bps = bitspersamp;
#if 0 // {
  return 1;
#else // } {
  return PlayerDispatchSync(&player,0,PlayerOpen,srate,numchan,bps);
#endif // }
}

void close()
{
  DPRINTF(player.options.common.nDebug,0,"%s\n",__func__);
  PlayerDispatchSync(&player,0,PlayerClose);
}

int write(char *buf, int len)
{
#if 0 // {
  writtentime += len;
  return 0;
#else // } {
  DPRINTF(player.options.common.nDebug,3,"%s\n",__func__);

  if (PlayerDispatchSync(&player,0,PlayerWrite,buf,len)<0) {
    DMESSAGEV(player.options.common.nDebug,"%s",__func__);
    return 1;
  }
  else {
#if 1 // {
  DPUTS(player.options.common.nDebug,3,"  end\n");
#endif // }
    writtentime += len;
    return 0;
  }
#endif // }
}

int canwrite()
{
  int bytes;

  DPRINTF(player.options.common.nDebug,3,"%s\n",__func__);
  bytes=PlayerDispatchSync(&player,0,PlayerCanWrite);
  DPRINTF(player.options.common.nDebug,3,"  bytes: %d\n",bytes);

  return bytes;
}

int isplaying()
{
#if 0 // {
  return 0;
#else // } {
  int playing;

  DPRINTF(player.options.common.nDebug,3,"%s\n",__func__);
  playing=PlayerDispatchSync(&player,0,PlayerIsPlaying);
  DPRINTF(player.options.common.nDebug,3,"  playing: %d\n",playing);

  if (!playing)
    reset();

  return playing;
#endif // }
}

int pause(int pause)
{
#if 0 // {
  int t=last_pause;
  if (!last_pause && pause) { w_offset+=GetTickCount()-start_t;  writtentime=0; }
  if (last_pause && !pause) { start_t=GetTickCount(); }
  last_pause=pause;
  return t;
#else // } {
#if defined (YASAPI_DEBUG) // {
  int debug=player.options.common.nDebug;
#endif // }
  int t=last_pause;
  int time;

  DPRINTF(debug,0,"%s (%d)\n",__func__,pause);

  if (last_pause!=pause) {
    time=PlayerDispatchSync(&player,0,PlayerPause,pause);

    if (pause) {
      w_offset+=GETTIME(time)-start_t;
      writtentime=0;
    }
    else
      start_t=GETTIME(time);

    last_pause=pause;
  }

  return t;
#endif // }
}

void setvolume(int volume)
{
  if (player.options.common.bVolume&&-666!=volume) {
    player.base.qVolume=(double)volume/255;
    DPRINTF(player.options.common.nDebug,0,"%s: %.2f\n",
        __func__,player.base.qVolume);
  }
}

void setpan(int pan)
{
}

void flush(int t)
{
  int time=PlayerDispatchSync(&player,0,PlayerFlush);

  w_offset=t;
  start_t=GETTIME(time);
  writtentime=0;
}
  
int getoutputtime()
{
#if 0 // {
  if (last_pause)
    return w_offset;
  return GetTickCount()-start_t + w_offset;
#else // } {
  if (last_pause)
    return w_offset;
  return GETTIME(GetAudioTime())-start_t + w_offset;
#endif // }
}

int getwrittentime()
{
  int t=srate*numchan,l;
  int ms=writtentime;

  if (t) {
    l=ms%t;
    ms /= t;
    ms *= 1000;
    ms += (l*1000)/t;

    ms/=(bps/8);

    return ms + w_offset;
  }
  else
    return ms;
}

#if 0 // {
Out_Module out = {
  OUT_VER,
  "Nullsoft NULL Output Plug-In " PI_VER
  
  ,
  65,
  0, // hmainwindow
  0, // hdllinstance
  config,
  about,
  init,
  quit,
  open,
  close,
  write,
  canwrite,
  isplaying,
  pause,
  setvolume,
  setpan,
  flush,
  getoutputtime,
  getwrittentime
};

__declspec( dllexport ) Out_Module * winampGetOutModule()
{
  return &out;
}
#else // } {
__declspec( dllexport ) Out_Module * winampGetOutModule()
{
  static Out_Module out;

  if (0==out.version) {
    out.version
      // module version (OUT_VER)
      =OUT_VER;
    out.description
      // description of module, with version string
      =PI_LABEL;
    out.id
      // module id. each input module gets its own.
      // non-nullsoft modules should be >= 65536. 
      =YASAPI_WA_ID;
    out.hMainWindow
      // winamp's main window (filled in by winamp)
      =NULL;
    out.hDllInstance
      // DLL instance handle (filled in by winamp)
      =NULL;
    out.Config
      // configuration dialog 
      =config;
    out.About
      // about dialog
      =about;
    out.Init
      // called when loaded
      =init;
    out.Quit
      // called when unloaded
      =quit;
    out.Open
      // returns >=0 on success, <0 on failure
      //
      // NOTENOTENOTE: bufferlenms and prebufferms are ignored in most
      // if not all output plug-ins. 
      //    ... so don't expect the max latency returned to be what you
      // asked for.
      // returns max latency in ms (0 for diskwriters, etc)
      // bufferlenms and prebufferms must be in ms. 0 to use defaults. 
      // prebufferms must be <= bufferlenms
      // pass bufferlenms==-666 to tell the output plugin that it's clock
      // is going to be used to sync video
      // out_ds turns off silence-eating when -666 is passed
      =open;
    out.Close
      // close the ol' output device.
      =close;
    out.Write
      // 0 on success. Len == bytes to write (<= 8192 always). buf is
      //   straight audio data. 
      // 1 returns not able to write (yet). Non-blocking, always.
      =write;
    out.CanWrite
      // returns number of bytes possible to write at a given time. 
      // Never will decrease unless you call Write (or Close, heh)
      =canwrite;
    out.IsPlaying
      // non0 if output is still going or if data in buffers waiting to be
      // written (i.e. closing while IsPlaying() returns 1 would truncate
      // the song
      =isplaying;
    out.Pause
      // returns previous pause state
      =pause;
    out.SetVolume
      // volume is 0-255
      =setvolume;
    out.SetPan
      // pan is -128 to 128
      =setpan;
    out.Flush
      // flushes buffers and restarts output at time t (in ms) 
      // (used for seeking)
      =flush;
    out.GetOutputTime
      // returns played time in MS
      =getoutputtime;
    out.GetWrittenTime
      // returns time written in MS (used for synching up vis stuff)
      =getwrittentime;
  }

  return &out;
}
#endif // }
