/*
 * yasapi_options_common.c
 * Copyright (C) 2015-2016 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yasapi.h>

///////////////////////////////////////////////////////////////////////////////
static const Property gcaCommonPropertyList[]={
#if defined (YASAPI_DEBUG) // {
  { YASAPI_PROPERTY_DEBUG,&gcIntType,offsetof(OptionsCommon,nDebug),0 },
#endif // }
  { L"id",&gcStringType,offsetof(OptionsCommon,szId),YASAPI_ID_SIZE },
  { L"mono2stereo",&gcIntType,offsetof(OptionsCommon,bMono2Stereo),0 },
  { L"volume",&gcIntType,offsetof(OptionsCommon,bVolume),0 },
  { L"block_write",&gcIntType,offsetof(OptionsCommon,bWriteBlock),0 },
  { L"format_supported",&gcIntType,offsetof(OptionsCommon,bFormatSupported),0 },
  { L"audioclock",&gcIntType,offsetof(OptionsCommon,bAudioClock),0 },
  { YASAPI_PROPERTY_VIS,&gcIntType,offsetof(OptionsCommon,bVisualization),0 },
  { YASAPI_PROPERTY_PAGE,&gcIntType,offsetof(OptionsCommon,nPage),0 },
  { YASAPI_PROPERTY_POSX,&gcIntType,offsetof(OptionsCommon,nPosX),0 },
  { YASAPI_PROPERTY_POSY,&gcIntType,offsetof(OptionsCommon,nPosY),0 },
#if defined (YASAPI_DEBUG) // {
  { L"sleep",&gcIntType,offsetof(OptionsCommon,nSleep),0 },
#endif // }
  { NULL,NULL,0,0 }
};

///////////////////////////////////////////////////////////////////////////////
int OptionsCommonLoad(OptionsCommon *pOptions, const wchar_t *path,
    int debug)
{
  PropertyLoadConfig c;

  c.debug=debug;
  c.path=path;
  c.pData=pOptions;
  c.pDefault=OptionsCommonDefault();
  c.group=YASAPI_GROUP_COMMON;
  c.pfx=L"  ";
  PropertyLoad(gcaCommonPropertyList,&c);

  return 0;
}

void OptionsCommonSave(OptionsCommon *pOptions, const wchar_t *path,
    int debug)
{
  PropertySaveConfig c;

  c.debug=debug;
  c.path=path;
  c.pData=pOptions;
  c.group=YASAPI_GROUP_COMMON;
  PropertySave(gcaCommonPropertyList,&c);
}

///////////////////////////////////////////////////////////////////////////////
const OptionsCommon *OptionsCommonDefault(void)
{
  static int initialized;
  static OptionsCommon options;

  if (!initialized) {
    options.nDebug=0;
    options.bMono2Stereo=TRUE;
    options.bWriteBlock=TRUE;
    options.bFormatSupported=FALSE;
    options.bAudioClock=TRUE;
    options.bVisualization=TRUE;
    options.nPage=0;
    options.nPosX=-1;
    options.nPosY=-1;
#if defined (YASAPI_DEBUG) // {
    options.nSleep=5000;
#endif // }

    initialized=1;
  }

  return &options;
}
