/*
 * yasapi_ring.c
 * Copyright (C) 2015-2016 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yasapi.h>
#include <stdint.h>

DWORD RingCreate(Ring *pRing, SIZE_T dwSize, int debug)
{
  pRing->Copy=RingCopyMemory;
  pRing->pClient=pRing;

  // create ring buffer.
#if defined (YASAPI_GLOBAL_ALLOC) // {
  pRing->pRep=(LPSTR)GlobalAlloc(
    GMEM_FIXED,     // _In_  UINT uFlags,
    dwSize          // _In_  SIZE_T dwBytes
  );
#else // } {
  pRing->pRep=(LPSTR)malloc(dwSize);
#endif // }

  if (NULL==pRing->pRep) {
    DMESSAGE(debug,"allocating ring buffer");
    goto rep;
  }
  
  pRing->pMax=pRing->pRep+dwSize;
  RingReset(pRing);

  return dwSize;
// cleanup:
  //GlobalFree(pRing->pRep);
rep:
  return -1;
}

void RingDestroy(Ring *pRing)
{
#if defined (YASAPI_GLOBAL_ALLOC) // {
  GlobalFree(pRing->pRep);
#else // } {
  free(pRing->pRep);
#endif // }
}

void RingReset(Ring *pRing)
{
  pRing->pHead=pRing->pRep;
  pRing->pTail=pRing->pRep;
  pRing->dwWritten=0;
  pRing->dwAvailable=pRing->pMax-pRing->pRep;
}

SIZE_T RingWrite(Ring *pRing, LPCSTR pData, SIZE_T dwSize, SIZE_T n)
{
  SIZE_T dwCopySize;

  dwSize*=n;

  if (dwSize>pRing->dwAvailable)
    dwSize=pRing->dwAvailable;

  if (0<dwSize) {
    if (0<(pRing->pHead-pRing->pTail)||dwSize<=(pRing->pMax-pRing->pTail)) {
      if (NULL!=pData) {
        pRing->Copy(
          pRing->pClient,
          pRing->pTail,       // _In_  PVOID Destination,
          pData,              // _In_  const VOID *Source,
          dwSize              // _In_  SIZE_T Length
        );
      }

      pRing->pTail+=dwSize;
    }
    else {
      dwCopySize=pRing->pMax-pRing->pTail;

      if (0<dwCopySize&&NULL!=pData) {
        pRing->Copy(
          pRing->pClient,
          pRing->pTail,       // _In_  PVOID Destination,
          pData,              // _In_  const VOID *Source,
          dwCopySize          // _In_  SIZE_T Length
        );

        pData+=dwCopySize/n;
      }

      dwCopySize=dwSize-dwCopySize;

      if (0<dwCopySize&&NULL!=pData) {
        pRing->Copy(
          pRing->pClient,
          pRing->pRep,        // _In_  PVOID Destination,
          pData,              // _In_  const VOID *Source,
          dwCopySize          // _In_  SIZE_T Length
        );
      }

      pRing->pTail=pRing->pRep+dwCopySize;
    }

    pRing->dwWritten+=dwSize;
    pRing->dwAvailable-=dwSize;
  }

  return dwSize;
}

#if 0 // {
SIZE_T RingRead(Ring *pRing, LPSTR pData, SIZE_T dwSize)
{
  SIZE_T dwCopySize;

  if (dwSize>pRing->dwWritten)
    dwSize=pRing->dwWritten;

  if (0<dwSize) {
    if (0<(pRing->pTail-pRing->pHead)||dwSize<=(pRing->pMax-pRing->pHead)) {
      if (NULL!=pData) {
        CopyMemory(
          pData,              // _In_  PVOID Destination,
          pRing->pHead,       // _In_  const VOID *Source,
          dwSize              // _In_  SIZE_T Length
        );
      }

      pRing->pHead+=dwSize;
    }
    else {
      dwCopySize=pRing->pMax-pRing->pHead;

      if (0<dwCopySize&&NULL!=pData) {
        CopyMemory(
          pData,              // _In_  PVOID Destination,
          pRing->pHead,       // _In_  const VOID *Source,
          dwCopySize          // _In_  SIZE_T Length
        );

        pData+=dwCopySize;
      }

      dwCopySize=dwSize-dwCopySize;

      if (0<dwCopySize&&NULL!=pData) {
        CopyMemory(
          pData,              // _In_  PVOID Destination,
          pRing->pRep,        // _In_  const VOID *Source,
          dwCopySize          // _In_  SIZE_T Length
        );
      }

      pRing->pHead=pRing->pRep+dwCopySize;
    }

    pRing->dwAvailable+=dwSize;
    pRing->dwWritten-=dwSize;
  }

  return dwSize;
}
#else // } {
SIZE_T RingReadEx(Ring *pRing, LPSTR pData, SIZE_T dwSize, UINT uFlags)
{
  SIZE_T dwCopySize;

  if (dwSize>pRing->dwWritten)
    dwSize=pRing->dwWritten;

  if (0<dwSize) {
    if (0<(pRing->pTail-pRing->pHead)||dwSize<=(pRing->pMax-pRing->pHead)) {
      if (NULL!=pData&&(RING_COPY&uFlags)) {
        CopyMemory(
          pData,              // _In_  PVOID Destination,
          pRing->pHead,       // _In_  const VOID *Source,
          dwSize              // _In_  SIZE_T Length
        );
      }

      if (RING_COMMIT&uFlags)
        pRing->pHead+=dwSize;
    }
    else {
      dwCopySize=pRing->pMax-pRing->pHead;

      if (0<dwCopySize&&NULL!=pData) {
        if (RING_COPY&uFlags) {
          CopyMemory(
            pData,            // _In_  PVOID Destination,
            pRing->pHead,     // _In_  const VOID *Source,
            dwCopySize        // _In_  SIZE_T Length
          );
        }

        pData+=dwCopySize;
      }

      dwCopySize=dwSize-dwCopySize;

      if (0<dwCopySize&&NULL!=pData&&(RING_COPY&uFlags)) {
        CopyMemory(
          pData,              // _In_  PVOID Destination,
          pRing->pRep,        // _In_  const VOID *Source,
          dwCopySize          // _In_  SIZE_T Length
        );
      }

      if (RING_COMMIT&uFlags)
        pRing->pHead=pRing->pRep+dwCopySize;
    }

    if (RING_COMMIT&uFlags) {
      pRing->dwAvailable+=dwSize;
      pRing->dwWritten-=dwSize;
    }
  }

  return dwSize;
}

SIZE_T RingRead(Ring *pRing, LPSTR pData, SIZE_T dwSize)
{
  return RingReadEx(pRing,pData,dwSize,RING_COPY|RING_COMMIT);
}
#endif // }

void RingCopyMemory(PVOID *Client, PVOID Destination, const VOID *Source,
    SIZE_T Length)
{
  CopyMemory(
    Destination,              // _In_  PVOID Destination,
    Source,                   // _In_  const VOID *Source,
    Length                    // _In_  SIZE_T Length
  );
}
